

# from spectramanipulator.spectrum import Spectrum, SpectrumList
# from spectramanipulator.settings import Settings
# from spectramanipulator.logger import Logger
# from spectramanipulator.console import Console
#
# __all__ = ['Spectrum', 'SpectrumList', 'Settings', 'Logger', 'Console']
#


#  Spectra Manipulator version
__version__ = "0.1.1"

